import json 
import requests
from time import sleep

class LongRunningOperation:
    """Class to represent a workspace in Microsoft Fabric"""

    def __init__(self, operation_id, auth) -> None:
        self.operation_id = operation_id
        self.auth = auth

        self.state = self.get_operation_state()["status"]


    def get_operation_results(self):
        """Get the results of an operation"""
        url = f"https://api.fabric.microsoft.com/v1/operations/{self.operation_id}/result"

        for _ in range(10):
            response = requests.get(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error getting operation results: {response.text}")
            break

        print(json.loads(response.text))
        return json.loads(response.text)
    
    def get_operation_state(self):
        """Get the state of an operation"""
        url = f"https://api.fabric.microsoft.com/v1/operations/{self.operation_id}"

        for _ in range(10):
            response = requests.get(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error getting operation state: {response.text}")
            break

        print(json.loads(response.text))
        return json.loads(response.text)    
    
    def wait_for_completion(self):
        """Wait for the operation to complete"""
        max_iter = 20
        while self.state not in ('Succeeded', 'Failed'):
            self.state = self.get_operation_state()["status"]
            sleep(3)
            if max_iter == 0:
                raise Exception("Operation did not complete after 60 seconds")
            max_iter -= 1
        return self.state
    

def check_long_running_operation(headers, auth):
    """Check the status of a long running operation"""
    location = headers.get('Location', None)
    operation_id = headers.get('x-ms-operation-id', None)
    if location:
        operation_id = location.split("/")[-1]
    
    if not operation_id:
        print("Operation initiated, no operation id found")
        return None
    else:
        print("Operation initiated, waiting for completion")
        lro = LongRunningOperation(operation_id=operation_id, auth=auth)
        lro.wait_for_completion()
        print("Operation completed")
    lro = LongRunningOperation(operation_id, auth)
    return lro.wait_for_completion()


import requests
import json
from time import sleep

from msfabricpysdkcore.capacity import Capacity
from msfabricpysdkcore.client import FabricClient
from msfabricpysdkcore.workspace import Workspace

class FabricClientCore(FabricClient):
    """FabricClientCore class to interact with Fabric Core APIs"""

    def __init__(self, tenant_id = None, client_id = None, client_secret = None) -> None:
        """Initialize FabricClientCore object"""
        super().__init__(tenant_id, client_id, client_secret)


    def list_workspaces(self, continuationToken = None):
        """List all workspaces in the tenant"""

        url = "https://api.fabric.microsoft.com/v1/workspaces"
        if continuationToken:
            url = f"{url}?continuationToken={continuationToken}"

        for _ in range(10):
            response = requests.get(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error listing workspaces: {response.status_code},  {response.text}")
            break
        resp_dict = json.loads(response.text)
        ws_list = resp_dict["value"]
        ws_list = [Workspace.from_dict(ws, auth=self.auth) for ws in ws_list]
      
        if "continuationToken" in resp_dict:
            ws_list_next = self.list_workspaces(continuationToken=resp_dict["continuationToken"])
            ws_list.extend(ws_list_next)

        return ws_list
    
    def get_workspace_by_name(self, name):
        """Get workspace by name"""
        ws_list = self.list_workspaces()
        for ws in ws_list:
            if ws.display_name == name:
                return ws
            
    def get_workspace_by_id(self, id):
        """Get workspace by id"""
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{id}"


        for _ in range(10):
            response = requests.get(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error getting workspace: {response.status_code} {response.text}")
            break
        ws_dict = json.loads(response.text)
        ws = Workspace.from_dict(ws_dict, auth=self.auth)

        return ws

    
    def get_workspace(self, id = None, name = None):
        """Get workspace by id or name"""
        if id:
            return self.get_workspace_by_id(id)
        if name:
            return self.get_workspace_by_name(name)
        raise ValueError("Either id or name must be provided")
        
    def get_workspace_role_assignments(self, workspace_id):
        """Get role assignments for a workspace"""
        ws = self.get_workspace_by_id(workspace_id)
        return ws.get_role_assignments()
    
    def create_workspace(self, display_name, capacity_id = None, description = None, exists_ok = True):
        """Create a workspace"""
        body = dict()
        body["displayName"] = display_name
        if capacity_id:
            body["capacityId"] = capacity_id
        if description:
            body["description"] = description
        
        url = "https://api.fabric.microsoft.com/v1/workspaces"

        for _ in range(10):
            response = requests.post(url=url, headers=self.auth.get_headers(), json=body)
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            ws_dict = json.loads(response.text)
            if response.status_code not in (201, 429):
                if "errorCode" in ws_dict and ws_dict["errorCode"] == "WorkspaceNameAlreadyExists" and exists_ok:
                    return self.get_workspace_by_name(display_name)
                else:
                    print(response.status_code)
                    print(response.text)
                    raise Exception(f"Error creating workspace: {response.text}")
            break

        ws = Workspace.from_dict(ws_dict, auth=self.auth)
        return ws
    
    def delete_workspace(self, workspace_id = None, display_name = None):
        """Delete a workspace"""
        if workspace_id is None and display_name is None:
            raise ValueError("Either workspace_id or display_name must be provided")
        ws = self.get_workspace(id = workspace_id, name = display_name)
        reponse = ws.delete()
        return reponse
    
    def add_workspace_role_assignment(self, workspace_id, role, principal):
        """Add a role assignment to a workspace"""
        ws = self.get_workspace_by_id(workspace_id)
        return ws.add_role_assignment(role, principal)
    
    def delete_workspace_role_assignment(self, workspace_id, principal_id):
        """Delete a role assignment from a workspace"""
        ws = self.get_workspace_by_id(workspace_id)
        return ws.delete_role_assignment(principal_id)
    
    def update_workspace(self, workspace_id, display_name = None, description = None):
        """Update a workspace"""
        ws = self.get_workspace_by_id(workspace_id)
        return ws.update(display_name, description)
    
    def update_workspace_role_assignment(self, workspace_id, role, principal_id):
        """Update a role assignment for a workspace"""
        ws = self.get_workspace_by_id(workspace_id)
        return ws.update_role_assignment(role, principal_id)

    def assign_to_capacity(self, workspace_id, capacity_id):
        """Assign a workspace to a capacity"""
        ws = self.get_workspace_by_id(workspace_id)
        return ws.assign_to_capacity(capacity_id)
    
    def unassign_from_capacity(self, workspace_id):
        """Unassign a workspace from a capacity"""
        ws = self.get_workspace_by_id(workspace_id)
        return ws.unassign_from_capacity()
 
    def list_capacities(self, continuationToken = None):
        """List all capacities in the tenant"""
        url = "https://api.fabric.microsoft.com/v1/capacities"

        if continuationToken:
            url = f"{url}?continuationToken={continuationToken}"

        for _ in range(10):
            response = requests.get(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error listing capacities: {response.text}")
            break

        resp_dict = json.loads(response.text)
        items = resp_dict["value"]

        if "continuationToken" in resp_dict:
            cap_list_next = self.list_capacities(continuationToken=resp_dict["continuationToken"])
            items.extend(cap_list_next)

        items = json.loads(response.text)["value"]
        items = [Capacity.from_dict(i) for i in items]
        return items

    
    def create_item(self, workspace_id, display_name, type, definition = None, description = None):
        """Create an item in a workspace"""
        ws = self.get_workspace_by_id(workspace_id)
        
        return ws.create_item(display_name = display_name,
                              type = type,
                              definition = definition,
                              description = description)

    def get_item(self, workspace_id = None, 
                  item_id = None, workspace_name = None, item_name = None, item_type = None):
        """Get an item from a workspace"""
        ws = self.get_workspace(id = workspace_id, name = workspace_name)
        return ws.get_item(item_id = item_id, item_name = item_name, item_type = item_type)

    def delete_item(self, workspace_id, item_id):
        """Delete an item from a workspace"""
        ws = self.get_workspace_by_id(workspace_id)
        return ws.delete_item(item_id)

    def list_items(self, workspace_id):
        """List items in a workspace"""
        ws = self.get_workspace_by_id(workspace_id)
        return ws.list_items()
    
    def get_item_definition(self, workspace_id, item_id):
        """Get the definition of an item"""
        ws = self.get_workspace_by_id(workspace_id)
        return ws.get_item_definition(item_id)
    
    def update_item(self, workspace_id, item_id, display_name = None, description = None):
        """Update an item in a workspace"""
        ws = self.get_workspace_by_id(workspace_id)
        return ws.get_item(item_id).update(display_name, description)
    
    def update_item_definition(self, workspace_id, item_id, definition):
        """Update the definition of an item"""
        ws = self.get_workspace_by_id(id=workspace_id)
        return ws.get_item(item_id=item_id).update_definition(definition=definition)
    
    def create_shortcut(self, workspace_id, item_id, path, name, target):
        ws = self.get_workspace_by_id(id=workspace_id)
        return ws.get_item(item_id=item_id).create_shortcut(path=path, name=name, target=target)
    
    def get_shortcut(self, workspace_id, item_id, path, name):
        ws = self.get_workspace_by_id(id=workspace_id)
        return ws.get_item(item_id=item_id).get_shortcut(path=path, name=name)
    
    def delete_shortcut(self, workspace_id, item_id, path, name):
        ws = self.get_workspace_by_id(id=workspace_id)
        return ws.get_item(item_id=item_id).delete_shortcut(path=path, name=name)
      
    def get_item_job_instance(self, workspace_id, item_id, job_instance_id):
        """Get a job instance for an item"""
        ws = self.get_workspace_by_id(id=workspace_id)
        return ws.get_item(item_id=item_id).get_item_job_instance(job_instance_id=job_instance_id)
    
    def run_on_demand_item_job(self, workspace_id, item_id, job_type, execution_data = None):
        """Run an on demand job for an item"""
        ws = self.get_workspace_by_id(id=workspace_id)
        return ws.get_item(item_id=item_id).run_on_demand_item_job(job_type=job_type, execution_data=execution_data)
    
    def cancel_item_job_instance(self, workspace_id, item_id, job_instance_id):
        """Cancel a job instance for an item"""
        ws = self.get_workspace_by_id(id=workspace_id)
        return ws.get_item(item_id=item_id).get_item_job_instance(job_instance_id=job_instance_id).cancel()
    
    def commit_to_git(self, workspace_id,mode, comment=None, items=None, workspace_head=None):
        """Commit changes to git"""
        ws = self.get_workspace_by_id(id=workspace_id)
        return ws.commit_to_git(mode=mode, comment=comment, items=items, workspace_head=workspace_head)
    
    def git_connect(self, workspace_id, git_provider_details):
        """Connect to git"""
        ws = self.get_workspace_by_id(id=workspace_id)
        return ws.git_connect(git_provider_details=git_provider_details)
    
    def git_disconnect(self, workspace_id):
        """Disconnect from git"""
        ws = self.get_workspace_by_id(id=workspace_id)
        return ws.git_disconnect()
    
    def git_get_connection(self, workspace_id):
        """Get git connection details"""
        ws = self.get_workspace_by_id(id=workspace_id)
        return ws.git_get_connection()
    
    def git_get_status(self, workspace_id):
        """Get git status"""
        ws = self.get_workspace_by_id(id=workspace_id)
        return ws.git_get_status()
    
    def git_initialize_connection(self, workspace_id, initialization_strategy):
        """Initialize git"""
        ws = self.get_workspace_by_id(id=workspace_id)
        return ws.git_initialize_connection(initialization_strategy=initialization_strategy)
    
    def update_from_git(self, workspace_id, remote_commit_hash, conflict_resolution = None, options = None, workspace_head = None):
        """Update workspace from git"""
        ws = self.get_workspace_by_id(id=workspace_id)
        return ws.update_from_git(remote_commit_hash=remote_commit_hash,
                                  conflict_resolution=conflict_resolution, 
                                  options=options, 
                                  workspace_head=workspace_head)
    
    def get_capacity(self, capacity_id = None, capacity_name = None):
        """Get a capacity
        
        Args:
            capacity_id (str): The ID of the capacity
            capacity_name (str): The name of the capacity
            
        Returns:
            Capacity: The capacity object
            
        Raises:
            ValueError: If no capacity is found
        """
        if capacity_id is None and capacity_name is None:
            raise ValueError("Either capacity_id or capacity_name must be provided")
        caps = self.list_capacities()
        for cap in caps:
            if capacity_id and cap.id == capacity_id:
                return cap
            if capacity_name and cap.display_name == capacity_name:
                return cap
        raise ValueError("No capacity found") 
    
    def list_tables(self, workspace_id, item_id):
        ws = self.get_workspace_by_id(workspace_id)
        return ws.list_tables(item_id=item_id)
    
    def load_table(self, workspace_id, item_id, table_name, path_type, relative_path,
                    file_extension = None, format_options = None,
                    mode = None, recursive = None, wait_for_completion = True):
        ws = self.get_workspace_by_id(workspace_id)
        return ws.load_table(item_id, table_name, path_type, relative_path,
                    file_extension, format_options,
                    mode, recursive, wait_for_completion)



import json 
import requests
from time import sleep

from msfabricpysdkcore.onelakeshortcut import OneLakeShortcut
from msfabricpysdkcore.job_instance import JobInstance
from msfabricpysdkcore.long_running_operation import check_long_running_operation


class Item:
    """Class to represent a item in Microsoft Fabric"""

    def __init__(self, id, display_name, type, workspace_id, auth, properties = None, definition=None, description="") -> None:
        
        self.id = id
        self.display_name = display_name
        self.description = description
        self.type = type
        self.definition = definition
        self.properties = properties
        self.workspace_id = workspace_id
        
        self.auth = auth

    def __str__(self) -> str:
        """Return a string representation of the workspace object"""
        dict_ = {
            'id': self.id,
            'display_name': self.display_name,
            'description': self.description,
            'type': self.type,
            'definition': self.definition,
            'workspace_id': self.workspace_id,
            'properties': self.properties
        }
        return json.dumps(dict_, indent=2)
    
    def __repr__(self) -> str:
        return self.__str__()
    
    def from_dict(item_dict, auth):
        """Create Item object from dictionary"""
        
        if item_dict['type'] == "Lakehouse":
            from msfabricpysdkcore.lakehouse import Lakehouse
            return Lakehouse(id=item_dict['id'], display_name=item_dict['displayName'], type=item_dict['type'], workspace_id=item_dict['workspaceId'],
                    properties=item_dict.get('properties', None),
                    definition=item_dict.get('definition', None), description=item_dict.get('description', ""), auth=auth)
        return Item(id=item_dict['id'], display_name=item_dict['displayName'], type=item_dict['type'], workspace_id=item_dict['workspaceId'],
                    properties=item_dict.get('properties', None),
                    definition=item_dict.get('definition', None), description=item_dict.get('description', ""), auth=auth)

    def delete(self):
        """Delete the workspace item"""
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.workspace_id}/items/{self.id}"
        for _ in range(10):
            response = requests.delete(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                print(self)
                raise Exception(f"Error deleting item: {response.text}")
            break

        return response.status_code
    
    def get_definition(self):
        """Get the definition of the item"""
        
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.workspace_id}/items/{self.id}/getDefinition"

        for _ in range(10):
            response = requests.post(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code == 202:
                check_long_running_operation( response.headers, self.auth)
                
            if response.status_code not in (200, 202, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error getting item definition: {response.text}")
            break
        
        print(response.text)
        print(response.status_code)
        return json.loads(response.text)
    
    def update(self, display_name = None, description = None):
        """Update the item"""
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.workspace_id}/items/{self.id}"

        payload = dict()
        if display_name:
            payload['displayName'] = display_name
        if description:
            payload['description'] = description

        for _ in range(10):
            response = requests.patch(url=url, headers=self.auth.get_headers(), json=payload)
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)

                raise Exception(f"Error updating item: {response.text}")
            break
        if display_name:
            self.display_name = payload['displayName']
        if description:
            self.description = payload['description']

        return self

    def update_definition(self, definition):
        """Update the item definition"""
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.workspace_id}/items/{self.id}/updateDefinition"
        payload = {
            'definition': definition
        }
        for _ in range(10):
            response = requests.post(url=url, headers=self.auth.get_headers(), json=payload)
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code == 202:
                check_long_running_operation( response.headers, self.auth)
            if response.status_code not in (200, 202, 429):
                print(response.status_code)
                print(response.text)

                raise Exception(f"Error updating item definition: {response.text}")
            break

        self.definition = payload['definition']
        return self
    
    def get_shortcut(self, path, name):
        """Get the shortcut in the item"""
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.workspace_id}/items/{self.id}/shortcuts/{path}/{name}"
        
        for _ in range(10):
            response = requests.get(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)

                raise Exception(f"Error getting shortcut: {response.text}")
            break

        shortcut_dict = json.loads(response.text)
        shortcut_dict['workspaceId'] = self.workspace_id
        shortcut_dict['itemId'] = self.id
        return OneLakeShortcut.from_dict(shortcut_dict,
                                         auth = self.auth)
    
    def create_shortcut(self, path, name, target):
        """Create a shortcut in the item"""
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.workspace_id}/items/{self.id}/shortcuts"

        body = {'name': name,
                'path': path,
                'target': target}

        for _ in range(10):
            response = requests.post(url=url, headers=self.auth.get_headers(), json=body)
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (201, 429):
                print(response.status_code)
                print(response.text)

                raise Exception(f"Error creating shortcut: {response.text}")
            break

        shortcut_dict = json.loads(response.text)
        shortcut_dict['workspaceId'] = self.workspace_id
        shortcut_dict['itemId'] = self.id
        return OneLakeShortcut.from_dict(shortcut_dict,
                                         auth = self.auth)
    
    def delete_shortcut(self, path, name):
        """Delete the shortcut in the item"""
        return self.get_shortcut(path, name).delete()
    

    def run_on_demand_item_job(self, job_type, execution_data = None):
        """Run an on demand job on the item"""

        # POST https://api.fabric.microsoft.com/v1/workspaces/{workspaceId}/items/{itemId}/jobs/instances?jobType={jobType}
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.workspace_id}/items/{self.id}/jobs/instances?jobType={job_type}"
        payload = {
            'executionData': execution_data
        }

        for _ in range(10):
            if execution_data:
                response = requests.post(url=url, headers=self.auth.get_headers(), json=payload)
            else:
                response = requests.post(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue

            if response.status_code not in (202, 429):
                print(response.status_code)
                print(response.text)

                raise Exception(f"Error running on demand job: {response.text}")
            break
        job_instance_id = response.headers["Location"].split("/")[-1]
        job_instance = self.get_item_job_instance(job_instance_id=job_instance_id)
        return job_instance

    def get_item_job_instance(self, job_instance_id):
        """Get the job instance of the item"""

        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.workspace_id}/items/{self.id}/jobs/instances/{job_instance_id}"
        for _ in range(10):
            response = requests.get(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)

                raise Exception(f"Error getting job instance: {response.text}")
            break

        job_dict = json.loads(response.text)
        job_dict['workspaceId'] = self.workspace_id
        job_dict['itemId'] = self.id
        return JobInstance.from_dict(job_dict, auth=self.auth)
    
    def cancel_item_job_instance(self, job_instance_id):
        """Cancel a job instance ofjob the item"""
        return self.get_item_job_instance(job_instance_id=job_instance_id).cancel()
        
    def list_tables(self):
        raise NotImplementedError("List tables only works on Lakehouse Items")
    
    def load_table(self, table_name, path_type, relative_path,
                    file_extension = None, format_options = None,
                    mode = None, recursive = None, wait_for_completion = True):
        raise NotImplementedError("Load table only works on Lakehouse Items")


import unittest
from datetime import datetime
from dotenv import load_dotenv
from time import sleep
from msfabricpysdkcore.coreapi import FabricClientCore

class TestFabricClientCore(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(TestFabricClientCore, self).__init__(*args, **kwargs)
        #load_dotenv()
        self.fc = FabricClientCore()
        self.workspace_id = "c3352d34-0b54-40f0-b204-cc964b1beb8d"

        datetime_str = datetime.now().strftime("%Y%m%d%H%M%S")
        self.item_name = "testitem" + datetime_str
        self.item_type = "Notebook"
    
    def test_item_end_to_end(self):

        item = self.fc.create_item(display_name=self.item_name, type=self.item_type, workspace_id=self.workspace_id) 
        self.assertEqual(item.display_name, self.item_name)
        self.assertEqual(item.type, self.item_type)
        self.assertEqual(item.workspace_id, self.workspace_id)
        self.assertEqual(item.description, "")

        item = self.fc.get_item(workspace_id=self.workspace_id, item_id=item.id)
        item_ = self.fc.get_item(workspace_id=self.workspace_id,
                                  item_name=self.item_name, item_type=self.item_type)
        self.assertEqual(item.id, item_.id)
        self.assertEqual(item.display_name, self.item_name)
        self.assertEqual(item.type, self.item_type)
        self.assertEqual(item.workspace_id, self.workspace_id)
        self.assertEqual(item.description, "")

        item_list = self.fc.list_items(workspace_id=self.workspace_id)
        self.assertTrue(len(item_list) > 0)

        item_ids = [item_.id for item_ in item_list]
        self.assertIn(item.id, item_ids)

        self.fc.update_item(workspace_id=self.workspace_id, item_id=item.id, display_name=f"u{self.item_name}")
        item = self.fc.get_item(workspace_id=self.workspace_id, item_id=item.id)
        self.assertEqual(item.display_name, f"u{self.item_name}")

        status_code = self.fc.delete_item(workspace_id=self.workspace_id, item_id=item.id)

        self.assertAlmostEqual(status_code, 200)

    def test_lakehouse(self):

        lakehouse = self.fc.get_item(workspace_id=self.workspace_id, item_name="lakehouse1", item_type="Lakehouse")
        item_id = lakehouse.id
        date_str = datetime.now().strftime("%Y%m%d%H%M%S")
        table_name = f"table{date_str}"


        status_code = self.fc.load_table(workspace_id=self.workspace_id, item_id=item_id, table_name=table_name, 
                                         path_type="File", relative_path="Files/folder1/titanic.csv")

        self.assertEqual(status_code, 202)

        table_list = self.fc.list_tables(workspace_id=self.workspace_id, item_id=item_id)
        table_names = [table["name"] for table in table_list]

        self.assertIn(table_name, table_names)

if __name__ == "__main__":
    unittest.main()


import json 
import requests
from time import sleep

from msfabricpysdkcore.long_running_operation import check_long_running_operation
from msfabricpysdkcore.item import Item

class Lakehouse(Item):
    """Class to represent a item in Microsoft Fabric"""

    def __init__(self, id, display_name, type, workspace_id, auth, properties = None, definition=None, description=""):
        super().__init__(id, display_name, type, workspace_id, auth, properties, definition, description)


    def list_tables(self, continuationToken = None):
        """List all tables in the lakehouse"""
        # GET https://api.fabric.microsoft.com/v1/workspaces/{workspaceId}/lakehouses/{lakehouseId}/tables
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.workspace_id}/lakehouses/{self.id}/tables"

        if continuationToken:
            url = f"{url}?continuationToken={continuationToken}"

        for _ in range(10):
            response = requests.get(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error listing tables: {response.status_code},  {response.text}")
            break
        resp_dict = json.loads(response.text)

        table_list = resp_dict["data"]

        if "continuationToken" in resp_dict and resp_dict["continuationToken"] is not None:
            table_list_next = self.list_tables(continuationToken=resp_dict["continuationToken"])
            table_list.extend(table_list_next)

        return table_list
    
    def load_table(self, table_name, path_type, relative_path,
                    file_extension = None, format_options = None,
                    mode = None, recursive = None, wait_for_completion = True):
        """Load a table in the lakehouse"""
        # POST https://api.fabric.microsoft.com/v1/workspaces/{workspaceId}/lakehouses/{lakehouseId}/tables/{tableName}/load
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.workspace_id}/lakehouses/{self.id}/tables/{table_name}/load"

        body = {
                "relativePath": relative_path,
                "pathType": path_type,
              }

        if file_extension:
            body["fileExtension"] = file_extension
        if format_options:
            body["formatOptions"] = format_options
        if mode:
            body["mode"] = mode
        if recursive:
            body["recursive"] = recursive

        for _ in range(10):
            response = requests.post(url=url, headers=self.auth.get_headers(), json=body)
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code == 202:
                if wait_for_completion:
                    success = self.check_if_table_is_created(table_name)
                    
                if not success:
                    print("Warning: Table not created after 3 minutes")
                else:
                    print("Table created")
            if response.status_code not in (202, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error loading table: {response.status_code},  {response.text}")
            break

        return response.status_code
    
    def check_if_table_is_created(self, table_name):
        """Check if the table is created"""
        for _ in range(60):
            table_names = [table["name"] for table in self.list_tables()]
            if table_name in table_names:
                return True
            
            sleep(3)
        return False
    


import os
from time import sleep

from msfabricpysdkcore.auth import FabricAuthClient, FabricServicePrincipal

class FabricClient():
    """FabricClient class to interact with Fabric API"""

    def __init__(self, tenant_id = None, client_id = None, client_secret = None) -> None:
        """Initialize FabricClient object"""
        self.tenant_id = tenant_id if tenant_id else os.getenv("FABRIC_TENANT_ID")
        self.client_id = client_id if client_id else os.getenv("FABRIC_CLIENT_ID")
        self.client_secret = client_secret if client_secret else os.getenv("FABRIC_CLIENT_SECRET")

        if self.client_id is None or self.client_secret is None or self.tenant_id is None:
            self.auth = FabricAuthClient()
        else:
            self.auth = FabricServicePrincipal(tenant_id = self.tenant_id,
                                               client_id = self.client_id, 
                                               client_secret = self.client_secret)

        self.scope = "https://api.fabric.microsoft.com/.default"


import unittest
from msfabricpysdkcore.coreapi import FabricClientCore
from datetime import datetime
from dotenv import load_dotenv

class TestFabricClientCore(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(TestFabricClientCore, self).__init__(*args, **kwargs)
        #load_dotenv()
        self.fc = FabricClientCore()
        self.workspace_id = "c3352d34-0b54-40f0-b204-cc964b1beb8d"

        self.lakehouse_target = "cb4ca0b5-b53b-4879-b206-a53c35cbff55"
        self.lakehouse_shortcut = "e2c09c89-bf97-4f71-bdeb-36338795ec36"

        datetime_str = datetime.now().strftime("%Y%m%d%H%M%S")
        self.shortcutname = "shortcut" + datetime_str
        self.path_target = "Files/folder1"
        self.path_shortcut = "Files/shortcutfolder"

        self.target = {'oneLake': {'itemId': self.lakehouse_target,
                                   'path': self.path_target,        
                                   'workspaceId': self.workspace_id}}

    def test_shortcut_end_to_end(self):

        item = self.fc.create_shortcut(workspace_id=self.workspace_id,
                                       item_id=self.lakehouse_shortcut,
                                       path=self.path_shortcut,
                                       name=self.shortcutname,
                                       target=self.target)
        self.assertEqual(item.name, self.shortcutname)
        self.assertEqual(item.path, self.path_shortcut)
        self.assertEqual(item.target, self.target)

        item = self.fc.get_shortcut(workspace_id=self.workspace_id,
                                    item_id=self.lakehouse_shortcut,
                                    path=self.path_shortcut,
                                    name=self.shortcutname)
        self.assertEqual(item.name, self.shortcutname)
        self.assertEqual(item.path, self.path_shortcut)
        self.assertEqual(item.target, self.target)

        status_code = self.fc.delete_shortcut(workspace_id=self.workspace_id,
                                             item_id=self.lakehouse_shortcut,
                                             path=self.path_shortcut,
                                             name=self.shortcutname)
        
        self.assertAlmostEqual(status_code, 200)

if __name__ == "__main__":
    unittest.main()


import requests
import json
import os
from time import sleep

from msfabricpysdkcore.admin_item import AdminItem
from msfabricpysdkcore.client import FabricClient
from msfabricpysdkcore.domain import Domain
from msfabricpysdkcore.admin_workspace import AdminWorkspace

class FabricClientAdmin(FabricClient):
    """FabricClientAdmin class to interact with Fabric Admin APIs"""

    def __init__(self, tenant_id = None, client_id = None, client_secret = None) -> None:
        """Initialize FabricClientAdmin object"""
        super().__init__(tenant_id, client_id, client_secret)

    
    def list_domains(self, nonEmptyOnly=False):
        """List all domains in the tenant
        
        Args:
            nonEmptyOnly (bool): Whether to list only non-empty domains
        Returns:
            list: List of Domain objects"""
        url = "https://api.fabric.microsoft.com/v1/admin/domains"
        if nonEmptyOnly:
            url = f"{url}?nonEmptyOnly=True"

        for _ in range(10):
            response = requests.get(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error listing domains: {response.text}")
            break

        resp_dict = json.loads(response.text)
        domains = resp_dict["domains"]

        domains = [Domain.from_dict(i, self.auth) for i in domains]
        return domains
    
    def get_domain_by_id(self, domain_id):
        """Method to get a domain by ID
        
        Args:
            domain_id (str): The ID of the domain
        Returns:
            Domain: The Domain object
        """
        url = f"https://api.fabric.microsoft.com/v1/admin/domains/{domain_id}"
        for _ in range(10):
            response = requests.get(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error getting domain: {response.text}")
            break

        domain_dict = json.loads(response.text)
        domain = Domain.from_dict(domain_dict, self.auth)
        return domain

    def get_domain_by_name(self, domain_name):
        """Method to get a domain by name
        
        Args:
            domain_name (str): The name of the domain
        Returns:
            Domain: The Domain object
        """
        domains = self.list_domains()
        for domain in domains:
            if domain.display_name == domain_name:
                return domain
        raise ValueError("Domain not found")
    
    def get_domain(self, domain_id = None, domain_name = None):
        """Get a domain by ID or name
        
        Args:
            domain_id (str): The ID of the domain
            domain_name (str): The name of the domain
        Returns:
            Domain: The Domain object
        """
        if domain_id:
            return self.get_domain_by_id(domain_id)
        if domain_name:
            return self.get_domain_by_name(domain_name)
        raise ValueError("Either domain_id or domain_name must be provided")
    
    def create_domain(self, display_name, description = None, parent_domain_id = None):
        """Method to create a domain
        
        Args:
            display_name (str): The display name of the domain
            description (str): The description of the domain
            parent_domain_id (str): The parent domain ID
        Returns:
            Domain: The Domain object
        """
        # POST https://api.fabric.microsoft.com/v1/admin/domains
        url = "https://api.fabric.microsoft.com/v1/admin/domains"
        body = {
            "displayName": display_name
        }
        if description:
            body["description"] = description
        if parent_domain_id:
            body["parentDomainId"] = parent_domain_id
        
        for _ in range(10):
            response = requests.post(url=url, headers=self.auth.get_headers(), json=body)
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 201, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error creating domain: {response.text}")
            break

        domain_dict = json.loads(response.text)
        domain = Domain.from_dict(domain_dict, self.auth)
        return domain
    
    def delete_domain(self, domain_id):
        """Method to delete a domain
        
        Args:
            domain_id (str): The ID of the domain
        Returns:
            status_code (int): The status code of the request
        """
        return self.get_domain_by_id(domain_id).delete()

    def update_domain(self, domain_id, description = None, display_name = None, contributors_scope = None):
        """Method to update a domain
        
        Args:
            domain_id (str): The ID of the domain
        Returns:
            Domain: The Domain object
        """
        domain = self.get_domain_by_id(domain_id)
        return domain.update(description = description, display_name = display_name, contributors_scope = contributors_scope)
    
    def list_domain_workspaces(self, domain_id, workspace_objects = False):
        """List workspaces in a domain
        
        Args:
            domain_id (str): The ID of the domain
        Returns:
            list: List of Workspace objects
        """
        domain = self.get_domain_by_id(domain_id)
        return domain.list_domain_workspaces(workspace_objects=workspace_objects)
    
    def assign_domain_workspaces_by_capacities(self, domain_id, capacities_ids, wait_for_completion=True):
        """Assign workspaces to a domain by capacities
        
        Args:
            domain_id (str): The ID of the domain
            capacities (list): The list of capacity IDs
            wait_for_completion (bool): Whether to wait for the operation to complete
        Returns:
            int: The status code of the response
        """
        domain = self.get_domain_by_id(domain_id)
        return domain.assign_workspaces_by_capacities(capacities_ids, wait_for_completion)
    
    def assign_domain_workspaces_by_ids(self, domain_id, workspace_ids):
        """Assign workspaces to a domain by workspace IDs
        
        Args:
            domain_id (str): The ID of the domain
            workspace_ids (list): The list of workspace IDs
            wait_for_completion (bool): Whether to wait for the operation to complete
        Returns:
            int: The status code of the response
        """
        domain = self.get_domain_by_id(domain_id)
        return domain.assign_workspaces_by_ids(workspace_ids)
    
    def assign_domains_workspaces_by_principals(self, domain_id, principals, wait_for_completion=True):
        """Assign workspaces to a domain by principals
        
        Args:
            domain_id (str): The ID of the domain
            principals (list): The list of principal IDs
            wait_for_completion (bool): Whether to wait for the operation to complete
        Returns:
            int: The status code of the response
        """
        domain = self.get_domain_by_id(domain_id)
        return domain.assign_workspaces_by_principals(principals=principals, wait_for_completion=wait_for_completion)
    
    def role_assignments_bulk_assign(self, domain_id, type, principals):
        """Assign a role to principals in bulk
        
        Args:
            domain_id (str): The ID of the domain
            type (str): The type of the role assignment
            principals (list): The list of principals
        Returns:
            int: The status code of the response
        """
        domain = self.get_domain_by_id(domain_id)
        return domain.role_assignments_bulk_assign(type, principals)
    
    def role_assignments_bulk_unassign(self, domain_id, type, principals):
        """Unassign a role from principals in bulk
        
        Args:
            domain_id (str): The ID of the domain
            type (str): The type of the role assignment
            principals (list): The list of principals
        Returns:
            int: The status code of the response
        """
        domain = self.get_domain_by_id(domain_id)
        return domain.role_assignments_bulk_unassign(type, principals)
    
    def unassign_all_domain_workspaces(self, domain_id):
        """Unassign all workspaces from a domain
        
        Args:
            domain_id (str): The ID of the domain
        Returns:
            int: The status code of the response
        """
        domain = self.get_domain_by_id(domain_id)
        return domain.unassign_all_workspaces()
    
    def unassign_domain_workspaces_by_ids(self, domain_id, workspace_ids):
        """Unassign workspaces from a domain by workspace IDs
        
        Args:
            domain_id (str): The ID of the domain
            workspace_ids (list): The list of workspace IDs
        Returns:
            int: The status code of the response
        """
        domain = self.get_domain_by_id(domain_id)
        return domain.unassign_workspaces_by_ids(workspace_ids=workspace_ids)
    
    def get_workspace(self, workspace_id):
        """Get a workspace by ID
        
        Args:
            workspace_id (str): The ID of the workspace
        Returns:
            Workspace: The Workspace object
        """

        url = f"https://api.fabric.microsoft.com/v1/admin/workspaces/{workspace_id}"
        for _ in range(10):
            response = requests.get(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error getting workspace: {response.text}")
            break

        workspace_dict = json.loads(response.text)
        workspace = AdminWorkspace.from_dict(workspace_dict, self.auth)
        return workspace
    
    def get_workspace_access_details(self, workspace_id):
        """Get the access details of the workspace
        
        Args:
            workspace_id (str): The ID of the workspace
        Returns:
            dict: The access details of the workspace
        """
        ws = self.get_workspace(workspace_id)
        return ws.get_workspace_access_details()
    
    def list_workspaces(self, capacity_id = None, name=None, state=None, type=None, continuationToken = None):
        """List all workspaces
        
        Args:
            capacity_id (str): The ID of the capacity
        Returns:
            list: List of Workspace objects
        """
        #GET https://api.fabric.microsoft.com/v1/admin/workspaces?type={type}&capacityId={capacityId}&name={name}&state={state}&continuationToken={continuationToken}
        url = "https://api.fabric.microsoft.com/v1/admin/workspaces"
        first_parameter = False
        if type:
            url = f"{url}?type={type}"
            first_parameter = True
        if capacity_id:
            if first_parameter:
                url = f"{url}&capacityId={capacity_id}"
            else:
                url = f"{url}?capacityId={capacity_id}"

        if name:
            if first_parameter:
                url = f"{url}&name={name}"
            else:
                url = f"{url}?name={name}"
        
        if state:
            if first_parameter:
                url = f"{url}&state={state}"
            else:
                url = f"{url}?state={state}"

        if continuationToken:
            if first_parameter:
                url = f"{url}&continuationToken={continuationToken}"
            else:
                url = f"{url}?continuationToken={continuationToken}"

        for _ in range(10):
            response = requests.get(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error listing workspaces: {response.text}")
            break

        resp_dict = json.loads(response.text)
        workspaces = resp_dict["workspaces"]

        workspaces = [AdminWorkspace.from_dict(i, self.auth) for i in workspaces]

        if "continuationToken" in resp_dict and resp_dict["continuationToken"] is not None:
            workspaces_next = self.list_workspaces(capacity_id=capacity_id, name=name, state=state, type=type,
                                                   continuationToken=resp_dict["continuationToken"])
            workspaces.extend(workspaces_next)

        return workspaces
        
    def list_items(self, workspace_id = None, capacity_id = None, type=None,
                   state=None, continuationToken = None):
        """List all items

        Returns:
            list: The list of items in the workspace
        """
        
        url = f"https://api.fabric.microsoft.com/v1/admin/items"
        first_parameter = False
        if workspace_id:
            url = f"{url}?workspaceId={workspace_id}"
            first_parameter = True
        if capacity_id:
            if first_parameter:
                url = f"{url}&capacityId={capacity_id}"
            else:
                url = f"{url}?capacityId={capacity_id}"
        if type:
            if first_parameter:
                url = f"{url}&type={type}"
            else:
                url = f"{url}?type={type}"
        if state:
            if first_parameter:
                url = f"{url}&state={state}"
            else:
                url = f"{url}?state={state}"
        if continuationToken:
            if first_parameter:
                url = f"{url}&continuationToken={continuationToken}"
            else:
                url = f"{url}?continuationToken={continuationToken}"
                
        for _ in range(10):
            response = requests.get(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error listing items: {response.text}")
            break
        resp_dict = json.loads(response.text)
        items =  [AdminItem.from_dict(item, self.auth) for item in resp_dict["itemEntities"]]

        if "continuationToken" in items and items["continuationToken"] is not None:
            items_next = self.list_items(workspace_id=workspace_id, capacity_id=capacity_id, type=type,
                                         state=state, continuationToken=items["continuationToken"])
            items.extend(items_next)

        return items
    
    def get_item(self, item_id, workspace_id, type = None):
        """Get an item from the workspace
        
        Args:
            item_id (str): The ID of the item
            workspace_id (str): The ID of the workspace
            type (str): The type of the item
        Returns:
            AdminItem: The item object
        """
        ws = self.get_workspace(workspace_id)
        return ws.get_item(item_id, type)
    

    def get_tenant_settings(self):
        """Get the tenant settings
        
        Returns:
            dict: The tenant settings
        """
        url = "https://api.fabric.microsoft.com/v1/admin/tenantsettings"
        for _ in range(10):
            response = requests.get(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error getting tenant settings: {response.text}")
            break

        return json.loads(response.text)
    

    def get_capacities_tenant_settings_overrides(self, continuationToken = None):
        """Returns list of tenant setting overrides that override at the capacities
        
        Returns:
            list: The capacities tenant settings overrides
        """
        url = "https://api.fabric.microsoft.com/v1/admin/capacities/delegatedTenantSettingOverrides"
        if continuationToken:
            url = f"{url}?continuationToken={continuationToken}"
        
        for _ in range(10):
            response = requests.get(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error getting capacities tenant settings overrides: {response.text}")
            break

        resp_dict = json.loads(response.text)
        overrides = resp_dict["Overrides"]

        if "continuationToken" in resp_dict and resp_dict["continuationToken"] is not None:
            overrides_next = self.get_capacities_tenant_settings_overrides(continuationToken=resp_dict["continuationToken"])
            overrides.extend(overrides_next)

        return overrides
    
    def get_access_entities(self, user_id, type = None, continuationToken = None):
        """Get the access entities for a user
        
        Args:
            user_id (str): The ID of the user
            type (str): The type of the access entity
            continuationToken (str): The continuation token
        Returns:
            list: The list of access entities
        """

        url = f"https://api.fabric.microsoft.com/v1/admin/users/{user_id}/access"

        first_parameter = False
        if type:
            url = f"{url}?type={type}"
            first_parameter = True
        if continuationToken:
            if first_parameter:
                url = f"{url}&continuationToken={continuationToken}"
            else:
                url = f"{url}?continuationToken={continuationToken}"

        for _ in range(10):
            response = requests.get(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error getting access entities: {response.text}")
            break

        resp_dict = json.loads(response.text)
        access_entities = resp_dict["accessEntities"]

        if "continuationToken" in resp_dict and resp_dict["continuationToken"] is not None:
            access_entities_next = self.get_access_entities(user_id, type, continuationToken=resp_dict["continuationToken"])
            resp_dict["accessEntities"].extend(access_entities_next)
        
        return access_entities
    
    def get_item_access_details(self, workspace_id, item_id, type=None):
        """Get the access details of the item
        
        Args:
            workspace_id (str): The ID of the workspace
            item_id (str): The ID of the item
            type (str): The type of the item
        Returns:
            dict: The access details of the item
        """
        ws = self.get_workspace(workspace_id)
        item = ws.get_item(item_id, type)
        return item.get_item_access_details(type)


import json 
import requests
from time import sleep

class OneLakeShortcut:
    """Class to represent a onelake shortcut in Microsoft Fabric"""

    def __init__(self, name, path, workspace_id, item_id, target,
                  auth) -> None:
        
        self.name = name
        self.path = path
        self.target = target
        self.item_id = item_id
        self.workspace_id = workspace_id

        self.user_auth = auth

    def __str__(self) -> str:
        """Return a string representation of the workspace object"""
        dict_ = {
            'name': self.name,
            'path': self.path,
            'target': self.target,
            'item_id': self.item_id,
            'workspace_id': self.workspace_id,
        }
        return json.dumps(dict_, indent=2)
    
    def __repr__(self) -> str:
        return self.__str__()
    
    def from_dict(short_dict, auth):
        """Create OneLakeShortCut object from dictionary"""
        return OneLakeShortcut(name=short_dict['name'],
                               path=short_dict['path'], 
                               target=short_dict['target'], 
                               item_id=short_dict['itemId'], 
                               workspace_id=short_dict['workspaceId'], 
                               auth=auth)
    
    def delete(self):
        """Delete the shortcut"""

        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.workspace_id}/items/{self.item_id}/shortcuts/{self.path}/{self.name}"
        for _ in range(10):
            response = requests.delete(url=url, headers=self.user_auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error deleting shortcut: {response.text}")
            break

        return response.status_code


import json 
import requests
from time import sleep
from msfabricpysdkcore.long_running_operation import check_long_running_operation

class JobInstance:
    """Class to represent a job instance in Microsoft Fabric"""

    def __init__(self, id, item_id, workspace_id, auth, job_type, invoke_type, status, root_activity_id,
                 start_time_utc, end_time_utc, failureReason):

        self.id = id
        self.item_id = item_id
        self.workspace_id = workspace_id
        self.job_type = job_type
        self.invoke_type = invoke_type
        self.status = status
        self.root_activity_id = root_activity_id
        self.start_time_utc = start_time_utc
        self.end_time_utc = end_time_utc
        self.failureReason = failureReason
        
        self.auth = auth


    def __str__(self) -> str:
        """Return a string representation of the workspace object"""
        dict_ = {
            'id': self.id,
            'item_id': self.item_id,
            'workspace_id': self.workspace_id,
            'job_type': self.job_type,
            'invoke_type': self.invoke_type,
            'status': self.status,
            'root_activity_id': self.root_activity_id,
            'start_time_utc': self.start_time_utc,
            'end_time_utc': self.end_time_utc,
            'failureReason': self.failureReason
        }
        return json.dumps(dict_, indent=2)
    
    def __repr__(self) -> str:
        return self.__str__()
    
    def from_dict(job_dict, auth):
        """Create JobInstance object from dictionary"""
        return JobInstance(id=job_dict['id'], item_id=job_dict['itemId'], workspace_id=job_dict['workspaceId'],
                           job_type=job_dict['jobType'], invoke_type=job_dict['invokeType'], status=job_dict['status'],
                           root_activity_id=job_dict['rootActivityId'], start_time_utc=job_dict['startTimeUtc'],
                           end_time_utc=job_dict['endTimeUtc'], failureReason=job_dict['failureReason'], auth=auth)
    
    def cancel(self):
        """Cancel the job instance"""
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.workspace_id}/items/{self.item_id}/jobs/instances/{self.id}/cancel"
        for _ in range(10):
            response = requests.post(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue

            if response.status_code not in (202, 429):
                print(response.status_code)
                print(response.text)

                raise Exception(f"Error running on demand job: {response.text}")
            break

        return response.status_code


import unittest
from dotenv import load_dotenv
from msfabricpysdkcore import FabricClientAdmin

load_dotenv()

class TestFabricClientCore(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(TestFabricClientCore, self).__init__(*args, **kwargs)

                  
    def test_domains(self):
        fca = FabricClientAdmin()

        user_id = 'b4f4e299-e6e1-4667-886c-57e4a8dde1c2'

        # List workspaces
        ws = fca.list_workspaces(name="testworkspace")[0]

        self.assertEqual(ws.name, "testworkspace")

        # Get workspace
        ws_clone = fca.get_workspace(workspace_id=ws.id)

        self.assertEqual(ws.id, ws_clone.id)

        # Get workspace access details

        ws_access = fca.get_workspace_access_details(ws.id)
        principials = ws_access["accessDetails"]
        principials_ids = [p["principal"]["id"] for p in principials]
        self.assertIn(user_id, principials_ids)

        # Get access entities

        access_entities = fca.get_access_entities(user_id, type="Notebook")
        self.assertGreater(len(access_entities), 0)

        # Get tenant settings

        tenant_settings = fca.get_tenant_settings()
        self.assertGreater(len(tenant_settings["tenantSettings"]), 0)

        # Get capacity tenant settings overrides

        overrides = fca.get_capacities_tenant_settings_overrides()
        self.assertGreater(len(overrides), -1)

        # List items

        item_list = fca.list_items(workspace_id=ws.id)
        self.assertGreater(len(item_list), 0)

        # Get item

        item = fca.get_item(workspace_id=ws.id, item_id=item_list[0].id)
        self.assertEqual(item.id, item_list[0].id)

        # Get item access details

        item_access = fca.get_item_access_details(workspace_id=ws.id, item_id=item_list[0].id)
        principials = item_access["accessDetails"]

        principials_ids = [p["principal"]["id"] for p in principials]

        self.assertIn(user_id, principials_ids)





import json 
import requests
from time import sleep
from msfabricpysdkcore.item import Item
from msfabricpysdkcore.long_running_operation import check_long_running_operation

class Workspace:
    """Class to represent a workspace in Microsoft Fabric"""

    def __init__(self, id, display_name, description, type, auth, capacity_id = None) -> None:
        self.id = id
        self.display_name = display_name
        self.description = description
        self.type = type
        self.capacity_id = capacity_id

        self.auth = auth
        
    
    def from_dict(dict,  auth):
        """Create a Workspace object from a dictionary"""
        return Workspace(id=dict['id'], display_name=dict['displayName'], description=dict['description'], type=dict['type'], capacity_id=dict.get('capacityId', None),
                         auth=auth)
    
    def __str__(self) -> str:
        """Return a string representation of the workspace object"""
        dict_ = {
            'id': self.id,
            'display_name': self.display_name,
            'description': self.description,
            'type': self.type,
            'capacity_id': self.capacity_id
        }
        return json.dumps(dict_, indent=2)
    
    def __repr__(self) -> str:
        return self.__str__()
    
    def get_role_assignments(self):
        """Get role assignments for the workspace"""
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.id}/roleAssignments"

        for _ in range(10):
            response = requests.get(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error getting role assignments: {response.text}")
            break

        return json.loads(response.text)
    
    def delete(self):
        """Delete the workspace"""
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.id}"

        for _ in range(10):
            response = requests.delete(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error deleting workspace: {response.text}")
            break

        return response.status_code
    
    # function to add workpace role assignment
    def add_role_assignment(self, role, principal):
        """Add a role assignment to the workspace"""
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.id}/roleAssignments"

        payload = {
            'principal': principal,
            'role': role
        }

        for _ in range(10):
            response = requests.post(url=url, headers=self.auth.get_headers(), data=json.dumps(payload))
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error adding role assignments: {response.text}")
            break

        return response.status_code
    

    def delete_role_assignment(self, principal_id):
        """Delete a role assignment from the workspace"""
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.id}/roleAssignments/{principal_id}"

        for _ in range(10):
            response = requests.delete(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error deleting role assignments: {response.text}")
            break

        
        return response.status_code
    
    def update(self, display_name = None, description = None):
        """Update the workspace"""
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.id}"

        body = dict()
        if display_name:
            body["displayName"] = display_name
        if description:
            body["description"] = description


        for _ in range(10):
            response = requests.patch(url=url, headers=self.auth.get_headers(), json=body)
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error updating workspace: {response.text}")
            break

        assert response.status_code == 200
        if display_name:
            self.display_name = display_name
        if description:
            self.description = description

        return self

    def update_role_assignment(self, role, principal_id):
        """Update a role assignment in the workspace"""
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.id}/roleAssignments/{principal_id}"
        body = {
            'role': role
        }

        for _ in range(10):
            response = requests.patch(url=url, headers=self.auth.get_headers(), json=body)
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error updating role assignments: {response.text}")
            break

        return response.status_code
    
    def assign_to_capacity(self, capacity_id):
        """Assign the workspace to a capacity"""
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.id}/assignToCapacity"

        body = {
            'capacityId': capacity_id
        }


        for _ in range(10):
            response = requests.post(url=url, headers=self.auth.get_headers(), json=body)
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code == 202:
                check_long_running_operation( response.headers, self.auth)
            if response.status_code not in (202, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error assigning capacity: {response.text}")
            break

        assert response.status_code == 202
        self.capacity_id = capacity_id
        return response.status_code
    
    def unassign_from_capacity(self):
        """Unassign the workspace from a capacity"""
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.id}/unassignFromCapacity"

        for _ in range(10):
            response = requests.post(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue

            if response.status_code == 202:
                check_long_running_operation( response.headers, self.auth)
            if response.status_code not in (202, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error unassigning capacity: {response.text}")
            break

        assert response.status_code == 202
        self.capacity_id = None
        return response.status_code
    
    def create_item(self, display_name, type, definition = None, description = None):
        """Create an item in a workspace"""

        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.id}/items"

        body = {
            'displayName': display_name,
            'type': type,
            'definition': definition,
            'description': description
        }

        for _ in range(10):
            response = requests.post(url=url, headers=self.auth.get_headers(), json=body)
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code == 202:
                check_long_running_operation( response.headers, self.auth)
            if response.status_code not in (201, 202, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error creating item: {response.text}")
            break

        item_dict = json.loads(response.text)
        if item_dict is None:
            print("Item not returned by API, trying to get it by name")
            return self.get_item_by_name(display_name, type)        
        return Item.from_dict(item_dict, auth=self.auth)
        

    def get_item_by_name(self, item_name, item_type):
        """Get an item from a workspace by name"""
        ws_items = self.list_items()
        for item in ws_items:
            if item.display_name == item_name and item.type == item_type:
                return item    

    def get_item(self, item_id = None, item_name = None, item_type = None):
        # GET https://api.fabric.microsoft.com/v1/workspaces/{workspaceId}/items/{itemId}
        """Get an item from a workspace"""
        if item_id is None and item_name is not None and item_type is not None:
            return self.get_item_by_name(item_name, item_type)
        elif item_id is None:
            raise Exception("item_id or the combination item_name + item_type is required")
        
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.id}/items/{item_id}"

        for _ in range(10):
            response = requests.get(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error getting item: {response.text}")
            break
        
        item_dict = json.loads(response.text)
        return Item.from_dict(item_dict, auth=self.auth)

    def delete_item(self, item_id):
        """Delete an item from a workspace"""
        return self.get_item(item_id).delete()
  

    def list_items(self, continuationToken = None):
        """List items in a workspace"""
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.id}/items"

        if continuationToken:
            url = f"{url}?continuationToken={continuationToken}"

        for _ in range(10):
            response = requests.get(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error listing items: {response.text}")
            break
        
        resp_dict = json.loads(response.text)
        items = resp_dict["value"]
        items = [Item.from_dict(item, auth=self.auth) for item in items]

        if "continuationToken" in resp_dict:
            item_list_next = self.list_items(continuationToken=resp_dict["continuationToken"])
            items.extend(item_list_next)

        return items
    
    def get_item_definition(self, item_id):
        """Get the definition of an item from a workspace"""
        return self.get_item(item_id).get_definition()
    
    def update_item(self, item_id, display_name = None, description = None):
        """Update an item in a workspace"""
        return self.get_item(item_id=item_id).update(display_name=display_name, description=description)
    
    def update_item_definition(self, item_id, definition):
        """Update the definition of an item in a workspace"""
        return self.get_item(item_id=item_id).update_definition(definition=definition)
    
    def create_shortcut(self, item_id, path, name, target):
        return self.get_item(item_id=item_id).create_shortcut(path=path, name=name, target=target)
    
    def get_shortcut(self, item_id, path, name):
        return self.get_item(item_id=item_id).get_shortcut(path=path, name=name)
    
    def delete_shortcut(self, item_id, path, name):
        return self.get_item(item_id=item_id).delete_shortcut(path=path, name=name)
    
    def run_on_demand_item_job(self, item_id, job_type, execution_data = None):
        return self.get_item(item_id=item_id).run_on_demand_job(job_type=job_type, execution_data = execution_data)
    
    def get_item_job_instance(self, item_id, job_instance_id):
        return self.get_item(item_id=item_id).get_item_job_instance(job_instance_id=job_instance_id)
    
    def cancel_item_job_instance(self, item_id, job_instance_id):
        return self.get_item(item_id=item_id).cancel_item_job_instance(job_instance_id=job_instance_id)
    
    def git_connect(self, git_provider_details):
        """Connect git"""
        # POST https://api.fabric.microsoft.com/v1/workspaces/{workspaceId}/git/connect
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.id}/git/connect"

        payload = {
            'gitProviderDetails': git_provider_details
        }

        for _ in range(10):
            response = requests.post(url=url, headers=self.auth.get_headers(), json=payload)
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 204, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error connecting git: {response.text}")
            break
        return response.status_code

    def git_disconnect(self):
        """Disconnect git"""
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.id}/git/disconnect"

        for _ in range(10):
            response = requests.post(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 204, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error disconnecting git: {response.text}")
            break
        return response.status_code

    def git_initialize_connection(self, initialization_strategy):
    #        POST https://api.fabric.microsoft.com/v1/workspaces/{workspaceId}/git/initializeConnection
        """Initialize git connection"""
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.id}/git/initializeConnection"

        body = {'initializeGitConnectionRequest':initialization_strategy}
            
        for _ in range(10):
            response = requests.post(url=url, headers=self.auth.get_headers(), json=body)
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code == 202:
                check_long_running_operation( response.headers, self.auth)
            if response.status_code not in (200, 202, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error initializing connection: {response.text}")
            break
        return response.status_code
    
    def git_get_status(self):
        """Get git connection status"""
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.id}/git/status"

        for _ in range(10):
            response = requests.get(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code == 202:
                check_long_running_operation( response.headers, self.auth)
            if response.status_code not in (200, 202, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error getting git connection status: {response.text}")
            break
        return json.loads(response.text)
    
    def git_get_connection(self):
        """Get git connection info"""
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.id}/git/connection"

        for _ in range(10):
            response = requests.get(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error getting git connection info: {response.text}")
            break
        return json.loads(response.text)
    
    def commit_to_git(self, mode, comment=None, items=None, workspace_head=None):
        # POST https://api.fabric.microsoft.com/v1/workspaces/{workspaceId}/git/commitToGit

        """Commit to git"""
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.id}/git/commitToGit"

        body = {
            'mode': mode
        }

        if comment:
            body['comment'] = comment
        if items:
            body['items'] = items
        if workspace_head:
            body['workspaceHead'] = workspace_head

        for _ in range(10):
            response = requests.post(url=url, headers=self.auth.get_headers(), json=body)
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code == 202:
                check_long_running_operation( response.headers, self.auth)
            if response.status_code not in (200, 202, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error committing to git: {response.text}")
            break

        return response.status_code

    def update_from_git(self, remote_commit_hash, conflict_resolution = None, options = None, workspace_head = None):
        # POST https://api.fabric.microsoft.com/v1/workspaces/{workspaceId}/git/updateFromGit
        """Update from git"""
        url = f"https://api.fabric.microsoft.com/v1/workspaces/{self.id}/git/updateFromGit"

        body = {
            "remoteCommitHash" : remote_commit_hash
        }

        if conflict_resolution:
            body['conflictResolution'] = conflict_resolution
        if options:
            body['options'] = options
        if workspace_head:
            body['workspaceHead'] = workspace_head

        for _ in range(10):
            response = requests.post(url=url, headers=self.auth.get_headers(), json=body)
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code == 202:
                check_long_running_operation( response.headers, self.auth)

            if response.status_code not in (200, 202, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error updating from git: {response.text}")
            break

        return response.status_code
    
    def list_tables(self, item_id):
        return self.get_item(item_id=item_id).list_tables()
    
    def load_table(self, item_id, table_name, path_type, relative_path,
                    file_extension = None, format_options = None,
                    mode = None, recursive = None, wait_for_completion = True):
        return self.get_item(item_id).load_table(table_name, path_type, relative_path,
                    file_extension, format_options,
                    mode, recursive, wait_for_completion)



import requests
from abc import abstractmethod
from azure.identity import AzureCliCredential

class FabricAuth():
    """FabricAuth class to interact with Entra ID"""

    @abstractmethod
    def get_token(self):
        """Get token from Azure AD"""
        pass

    def get_headers(self):
        """Get headers for API requests"""
        access_token = self.get_token()
        headers = {
            'Authorization': 'Bearer ' + access_token,
            'Content-Type': 'application/json'
        }
        return headers


class FabricAuthClient(FabricAuth):
    """FabricAuthClient class to interact with Entra ID"""

    def __init__(self):
        print("Using Azure CLI for authentication")
        self.auth = AzureCliCredential()

    def get_token(self):
        """Get token from Azure AD"""
        token = self.auth.get_token("https://api.fabric.microsoft.com/.default")
        return token.token

class FabricServicePrincipal(FabricAuth):
    """FabricServicePrincipal class to interact with Entra ID"""

    def __init__(self, tenant_id, client_id, client_secret):
        print("Using Service Principal for authentication")

        self.tenant_id = tenant_id
        self.client_id = client_id
        self.client_secret = client_secret

        self.scope = "https://api.fabric.microsoft.com/.default"

    
    def get_token(self):
        """Get token from Azure AD"""
        # Get token from Azure AD
        url = f"https://login.microsoftonline.com/{self.tenant_id}/oauth2/v2.0/token"
        payload = {
            'grant_type': 'client_credentials',
            'client_id': f'{self.client_id}',
            'client_secret': f'{self.client_secret}',
            'scope': self.scope
        }
        response = requests.post(url, data=payload)
        access_token = response.json().get('access_token')
        return access_token
    


import json
from time import sleep

import requests
from msfabricpysdkcore.long_running_operation import check_long_running_operation

class Domain:
    """Class to represent a domain in Microsoft Fabric"""

    def __init__(self, id, display_name, description, parent_domain_id, contributors_scope, auth):
        """Constructor for the Domain class
        
        Args:
            id (str): The ID of the domain
            display_name (str): The display name of the domain
            description (str): The description of the domain
            parent_domain_id (str): The parent domain ID of the domain
            contributors_scope (str): The contributors scope of the domain
        Returns:
            Domain: The Domain object created"""

        self.id = id
        self.display_name = display_name
        self.description = description
        self.parent_domain_id = parent_domain_id
        self.contributors_scope = contributors_scope

        self.auth = auth

    def __str__(self):
        """Method to return a string representation of the Domain object
        
        Returns:
            str: The string representation of the Domain object
        """
        dic = {
            'id': self.id,
            'display_name': self.display_name,
            'description': self.description,
            'parent_domain_id': self.parent_domain_id,
            'contributors_scope': self.contributors_scope
        }
        return json.dumps(dic, indent=2)
    
    def __repr__(self) -> str:
        return self.__str__()

    def from_dict(dic, auth):
        """Method to create a Domain object from a dictionary
        
        Args:
            dic (dict): The dictionary containing the domain information
        Returns:
            Domain: The Domain object created from the dictionary
        
        """
        if "display_name" not in dic:
            dic["display_name"] = dic["displayName"]
        if "parent_domain_id" not in dic:
            dic["parent_domain_id"] = dic["parentDomainId"]
        if "contributors_scope" not in dic:
            dic["contributors_scope"] = dic["contributorsScope"]
        return Domain(id=dic['id'], display_name=dic['display_name'],
                      description=dic['description'], parent_domain_id=dic['parent_domain_id'], 
                      contributors_scope=dic['contributors_scope'], auth=auth)

    def list_domain_workspaces(self, workspace_objects = False, continuationToken = None):
        """Method to list the workspaces in the domain
        
        Args:
            continuationToken (str): The continuation token to use for pagination
        Returns:
            list: The list of workspaces in the domain
        """
        if workspace_objects:
            from msfabricpysdkcore import FabricClientCore
            fc = FabricClientCore()
                    
        url = f"https://api.fabric.microsoft.com/v1/admin/domains/{self.id}/workspaces"
        if continuationToken:
            url = f"{url}?continuationToken={continuationToken}"

        for _ in range(10):
            response = requests.get(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error listing workspaces: {response.text}")
            break

        resp_dict = json.loads(response.text)
        workspaces = resp_dict["value"]

        if workspace_objects:
            workspaces = [fc.get_workspace_by_id(workspace["id"]) for workspace in workspaces]

        if "continuationToken" in resp_dict:
            workspaces_next = self.list_domain_workspaces(continuationToken=resp_dict["continuationToken"])
            workspaces.extend(workspaces_next)

        return workspaces
    
    def delete(self):
        """Method to delete the domain

        Returns:
            int: The status code of the response
        """
        # DELETE https://api.fabric.microsoft.com/v1/admin/domains/{domainId}
        url = f"https://api.fabric.microsoft.com/v1/admin/domains/{self.id}"
        for _ in range(10):
            response = requests.delete(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error deleting domain: {response.text}")
            break

        return response.status_code
    

    # PATCH https://api.fabric.microsoft.com/v1/admin/domains/{domainId}

    def update(self, description = None, display_name = None, contributors_scope = None):
        """Method to update the domain
        
        Args:
            description (str): The description of the domain
            display_name (str): The display name of the domain
            contributors_scope (str): The contributors scope of the domain
        Returns:
             Domain: The Domain object created from the dictionary
        """
        url = f"https://api.fabric.microsoft.com/v1/admin/domains/{self.id}"
        body = {}
        if description:
            body["description"] = description
        else:
            body["description"] = self.description

        if display_name:
            body["displayName"] = display_name
        else:
            body["displayName"] = self.display_name

        if contributors_scope:
            body["contributorsScope"] = contributors_scope
        else:
            body["contributorsScope"] = self.contributors_scope

        for _ in range(10):
            response = requests.patch(url=url, headers=self.auth.get_headers(), json=body)
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error updating domain: {response.text}")
            break

        self.description = body["description"]
        self.display_name = body["displayName"]
        self.contributors_scope = body["contributorsScope"]

        return self
    
    def assign_workspaces_by_capacities(self, capacities_ids, wait_for_completion=True):
        """Method to assign workspaces to the domain based on capacities_ids
        
        Args:
            capacities_ids (list): The list of capacitiy ids to assign workspaces
        Returns:
            int: The status code of the response
        """
        # POST https://api.fabric.microsoft.com/v1/admin/domains/{domainId}/assignWorkspacesByCapacities
        url = f"https://api.fabric.microsoft.com/v1/admin/domains/{self.id}/assignWorkspacesByCapacities"

        body = {
            "capacitiesIds": capacities_ids
        }

        for _ in range(10):
            response = requests.post(url=url, headers=self.auth.get_headers(), json=body)
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code == 202 and wait_for_completion:
                check_long_running_operation(response.headers, self.auth)
            if response.status_code not in (202, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error assigning workspaces by capacities: {response.text}")
            break

        return response.status_code


    def assign_workspaces_by_ids(self, workspaces_ids):

        """Method to assign workspaces to the domain based on workspaces_ids
        
        Args:
            workspaces_ids (list): The list of workspace ids to assign workspaces
        Returns:
            int: The status code of the response
        """
        url = f"https://api.fabric.microsoft.com/v1/admin/domains/{self.id}/assignWorkspaces"
        body = {
            "workspacesIds": workspaces_ids
        }

        for _ in range(10):
            response = requests.post(url=url, headers=self.auth.get_headers(), json=body)
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error assigning workspaces by ids: {response.text}")
            break

        return response.status_code
    

    def assign_workspaces_by_principals(self, principals, wait_for_completion=True):
        """Method to assign workspaces to the domain based on principals
        
        Args:
            principals (list): The list of principals to assign workspaces
        Returns:
            int: The status code of the response
        """
        url = f"https://api.fabric.microsoft.com/v1/admin/domains/{self.id}/assignWorkspacesByPrincipals"
        body = {
            "principals": principals
        }

        for _ in range(10):
            response = requests.post(url=url, headers=self.auth.get_headers(), json=body)
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code == 202 and wait_for_completion:
                check_long_running_operation(response.headers, self.auth)
            if response.status_code not in (202, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error assigning workspaces by principals: {response.text}")
            break

        return response.status_code
    
    # POST https://api.fabric.microsoft.com/v1/admin/domains/{domainId}/roleAssignments/bulkAssign

    def role_assignments_bulk_assign(self, type, principals):
        """Method to bulk assign role assignments
        
        Args:
            type (str): The type of the role assignment
            principals (list): The list of principals to assign the role
        Returns:
            int: The status code of the response
        """
        
        url = f"https://api.fabric.microsoft.com/v1/admin/domains/{self.id}/roleAssignments/bulkAssign"
        body = {
            "type": type,
            "principals": principals
        }

        for _ in range(10):
            response = requests.post(url=url, headers=self.auth.get_headers(), json=body)
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error bulk assigning role assignments: {response.text}")
            break

        return response.status_code
    

    def role_assignments_bulk_unassign(self, type, principals):
        """Method to bulk unassign role assignments
        
        Args:
            type (str): The type of the role assignment
            principals (list): The list of principals to unassign the role
        Returns:
            int: The status code of the response
        """
        
        url = f"https://api.fabric.microsoft.com/v1/admin/domains/{self.id}/roleAssignments/bulkUnassign"
        body = {
            "type": type,
            "principals": principals
        }

        for _ in range(10):
            response = requests.post(url=url, headers=self.auth.get_headers(), json=body)
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error bulk unassigning role assignments: {response.text}")
            break

        return response.status_code
    

    def unassign_all_workspaces(self):
        """Method to unassign all workspaces from the domain
        
        Returns:
            int: The status code of the response
        """
        url = f"https://api.fabric.microsoft.com/v1/admin/domains/{self.id}/unassignAllWorkspaces"

        for _ in range(10):
            response = requests.post(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error unassigning all workspaces: {response.text}")
            break

        return response.status_code
    
    def unassign_workspaces_by_ids(self, workspace_ids):
        """Method to unassign workspaces from the domain
        
        Args:
            workspace_ids (list): The list of workspace ids to unassign
        Returns:
            int: The status code of the response
        """
        url = f"https://api.fabric.microsoft.com/v1/admin/domains/{self.id}/unassignWorkspaces"
        body = {
            "workspacesIds": workspace_ids
        }

        for _ in range(10):
            response = requests.post(url=url, headers=self.auth.get_headers(), json=body)
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error unassigning workspaces by ids: {response.text}")
            break

        return response.status_code


import unittest
from msfabricpysdkcore.coreapi import FabricClientCore

class TestFabricClientCore(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(TestFabricClientCore, self).__init__(*args, **kwargs)
        #load_dotenv()
        self.fc = FabricClientCore()
        self.workspace_id = "c3352d34-0b54-40f0-b204-cc964b1beb8d"
        self.item_id = "7e38f344-81df-4805-83b6-b9cc16830500"


    def test_jobs_end_to_end(self):
        job = self.fc.run_on_demand_item_job(workspace_id=self.workspace_id,
                                            item_id=self.item_id,
                                            job_type="RunNotebook")
        
        self.assertEqual(job.item_id, self.item_id)
        self.assertEqual(job.workspace_id, self.workspace_id)
        self.assertEqual(job.job_type, "RunNotebook")
        self.assertIn(job.status, ["NotStarted", "InProgress"])
        self.assertEqual(job.invoke_type, "Manual")

        job2 = self.fc.get_item_job_instance(workspace_id=self.workspace_id,
                                        item_id=self.item_id,
                                        job_instance_id=job.id)
        
        self.assertEqual(job.id, job2.id)

        status_code = self.fc.cancel_item_job_instance(workspace_id=self.workspace_id,
                                                  item_id=self.item_id,
                                                  job_instance_id=job.id)

        self.assertEqual(status_code, 202)

if __name__ == "__main__":
    unittest.main()




import json
from time import sleep

import requests

from msfabricpysdkcore.admin_item import AdminItem 


class AdminWorkspace:
    """Class to represent a workspace in Microsoft Fabric"""

    def __init__(self, id, type, name, state, capacity_id, auth) -> None:
        """Constructor for the Workspace class

        Args:
            id (str): The ID of the workspace
            type (str): The type of the workspace
            name (str): The name of the workspace
            state (str): The state of the workspace
            capacity_id (str): The ID of the capacity
            auth (Auth): The Auth object
        Returns:
            Workspace: The Workspace object
        """
        self.id = id
        self.type = type
        self.name = name
        self.state = state
        self.capacity_id = capacity_id
        self.auth = auth


    def __str__(self) -> str:
        """Return a string representation of the workspace object
        
        Returns:
            str: The string representation of the workspace object
        """
        dict_ = {
            'id': self.id,
            'type': self.type,
            'name': self.name,
            'state': self.state,
            'capacity_id': self.capacity_id
        }
        return json.dumps(dict_, indent=2)

    
    def __repr__(self) -> str:
        return self.__str__()
    
    def from_dict(item_dict, auth):
        """Create Workspace object from dictionary

        Args:
            item_dict (dict): The dictionary representing the workspace
            auth (Auth): The Auth object
        Returns:
            Workspace: The Workspace object
        """
        return AdminWorkspace(
            id=item_dict['id'],
            type=item_dict['type'],
            name=item_dict['name'],
            state=item_dict['state'],
            capacity_id=item_dict['capacityId'],
            auth=auth
        )
    
    def get_workspace_access_details(self):
        """Get the access details of the workspace

        Returns:
            dict: The access details of the workspace
        """
        url = f"https://api.fabric.microsoft.com/v1/admin/workspaces/{self.id}/users"
           
        for _ in range(10):
            response = requests.get(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error getting workspace: {response.text}")
            break

        return json.loads(response.text)
        
    def get_item(self, item_id, type = None):
        """Get an item from the workspace
        
        Args:
            item_id (str): The ID of the item
            type (str): The type of the item
        Returns:
            AdminItem: The item object
        """
        url = f"https://api.fabric.microsoft.com/v1/admin/workspaces/{self.id}/items/{item_id}"
        if type:
            url += f"?type={type}"
        for _ in range(10):
            response = requests.get(url=url, headers=self.auth.get_headers())
            if response.status_code == 429:
                print("Too many requests, waiting 10 seconds")
                sleep(10)
                continue
            if response.status_code not in (200, 429):
                print(response.status_code)
                print(response.text)
                raise Exception(f"Error getting item: {response.text}")
            break
        item_dict = json.loads(response.text) 
        return AdminItem.from_dict(item_dict, self.auth)
    
    def get_item_access_details(self, item_id, type=None):
        """Get the access details of the item
        
        Args:
            item_id (str): The ID of the item
            type (str): The type of the item
        Returns:
            dict: The access details of the item
        """
        return self.get_item(item_id, type).get_item_access_details()
